"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
var visitor_1 = require("./visitor");
exports.plugin = function (schema, documents, config) {
    var imports = ['GraphQLResolveInfo'];
    var hasScalars = Object.values(schema.getTypeMap())
        .filter(function (t) { return t.astNode; })
        .some(graphql_1.isScalarType);
    if (hasScalars) {
        imports.push('GraphQLScalarType', 'GraphQLScalarTypeConfig');
    }
    var visitor = new visitor_1.TypeScriptResolversVisitor(config, schema);
    var header = "\nimport { " + imports.join(', ') + " } from 'graphql';\n\nexport type Resolver<Result, Parent = {}, Context = {}, Args = {}> = (\n  parent?: Parent,\n  args?: Args,\n  context?: Context,\n  info?: GraphQLResolveInfo\n) => Promise<Result> | Result;\n\nexport type SubscriptionSubscribeFn<Result, Parent, Context, Args> = (\n  parent?: Parent,\n  args?: Args,\n  context?: Context,\n  info?: GraphQLResolveInfo\n) => AsyncIterator<Result> | Promise<AsyncIterator<Result>>;\n\nexport type SubscriptionResolveFn<Result, Parent, Context, Args> = (\n  parent?: Parent,\n  args?: Args,\n  context?: Context,\n  info?: GraphQLResolveInfo\n) => Result | Promise<Result>;\n\nexport interface ISubscriptionResolverObject<Result, Parent, Context, Args> {\n  subscribe: SubscriptionSubscribeFn<Result, Parent, Context, Args>;\n  resolve?: SubscriptionResolveFn<Result, Parent, Context, Args>;\n}\n\nexport type SubscriptionResolver<Result, Parent = {}, Context = {}, Args = {}> =\n  | ((...args: any[]) => ISubscriptionResolverObject<Result, Parent, Context, Args>)\n  | ISubscriptionResolverObject<Result, Parent, Context, Args>;\n\nexport type TypeResolveFn<Types, Parent = {}, Context = {}> = (\n  parent?: Parent,\n  context?: Context,\n  info?: GraphQLResolveInfo\n) => Maybe<Types>;\n\nexport type NextResolverFn<T> = () => Promise<T>;\n\nexport type DirectiveResolverFn<Result = {}, Parent = {}, Context = {}, Args = {}> = (\n  next?: NextResolverFn<Result>,\n  parent?: Parent,\n  args?: Args,\n  context?: Context,\n  info?: GraphQLResolveInfo\n) => Result | Promise<Result>;\n";
    var printedSchema = graphql_1.printSchema(schema);
    var astNode = graphql_1.parse(printedSchema);
    var visitorResult = graphql_1.visit(astNode, { leave: visitor });
    var getRootResolver = visitor.getRootResolver, getAllDirectiveResolvers = visitor.getAllDirectiveResolvers, mappersImports = visitor.mappersImports;
    return mappersImports.concat([
        header
    ], visitorResult.definitions.filter(function (d) { return typeof d === 'string'; }), [
        getRootResolver(),
        getAllDirectiveResolvers()
    ]).join('\n');
};
//# sourceMappingURL=index.js.map